package pr_x.driver.text;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.IOException;
import java.io.Reader;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Calendar;
import java.util.Map;
import java.util.Vector;
import java.sql.*;
import java.net.*;
/** */
public class TextResultSet implements ResultSet {
    /** statement */
    private TextStatement st;
    /** SQL */
    private String sql;

    /** buffered reader */
    private BufferedReader bfr;
    /** the number of rows */
    private int size;
    /** Number of rows of this result set */
    private Vector rows = new Vector();
    /** Index to current row (0..size-1) */
    private int index = -1;
    /** type */
    private int type;
    /** concurrency */
    private int concurrency;
    /** fetch direction and size */
    private int fetchDirection, fetchSize;
    /** last column */
    private String lastColumn;

    /** constructor */
    TextResultSet(TextStatement st, String sql) throws SQLException {
	     this.st = st;
	     this.sql= sql;
	     String s;

	     System.out.println("sql="+sql+"  max="+st.getMaxRows());

	     TextConnection con = (TextConnection) st.getConnection();
	     bfr = con.getBufferedReader();

	     try {
	     	 while ((s = bfr.readLine()) != null) {
	         if (sql==null) rows.add(s);              // fetch all
	         else if(s.indexOf(sql)>=0) rows.add(s);  // select
	     	 	 if(rows.size()==st.getMaxRows()) break; 	// limit
	       }
	       bfr.close();
	     }
	     catch (IOException e) {e.printStackTrace();}
	     size = rows.size();
	     type = st.getResultSetType();
	     concurrency = st.getResultSetConcurrency();
	     fetchDirection = st.getFetchDirection();
	     fetchSize = st.getFetchSize();

    }
    // return the current row
    private String getCurrentRow() {
	     if (index < 0 || index >= size) return null;
	     return (String) rows.get(index);
    }
    // fill lastColumn with the specified column from the current row
    private void getColumn(int col) {
	     String row = getCurrentRow();
	     lastColumn = row;
    }
    // move to the next row
    public boolean next() throws SQLException {
	     if (index < size) index++;
	     return index < size;
    }
    // move to the previous row
    public boolean previous() throws SQLException {
	    if (type == ResultSet.TYPE_FORWARD_ONLY)
	      throw new TextResultSetException();
	    if (index >= 0) index--;
	    return index >= 0;
    }
    // move to the absolute position (row < 0 is from the end)
    public boolean absolute(int row) throws SQLException {
      if (type == ResultSet.TYPE_FORWARD_ONLY)
	       throw new TextResultSetException();
	     if (row > 0) index = row - 1;
	     else if (row < 0) index = size + row;
	     else throw new TextResultSetException("cannot move to 0th row");
	     return true;
    }
    // move to the relative position
    public boolean relative(int row) throws SQLException {
	    if (type == ResultSet.TYPE_FORWARD_ONLY)
	       throw new TextResultSetException();
	    int i = index + row;
	    if (i < 0 || i >= size) return false;
	    index = i;
	    return true;
    }
    public boolean isBeforeFirst() throws SQLException { return index < 0; }
    public boolean isAfterLast() throws SQLException { return index >= size; }
    public boolean isFirst() throws SQLException { return index == 0; }
    public boolean isLast() throws SQLException { return index == size - 1; }
    public void    beforeFirst() throws SQLException { index = -1; }
    public void    afterLast() throws SQLException { index = size; }
    public boolean first() throws SQLException { return absolute(1); }
    public boolean last() throws SQLException { return absolute(-1); }
    // return the number of current row (1..size)
    public int getRow() throws SQLException { return index + 1; }
    /**  close this result set */
    public void close() throws SQLException{
    	 try {bfr.close();} catch (Exception e) {}
    	 rows=null; bfr=null;
    }

    // find the specified column
    public int findColumn(String columnName) throws SQLException {
	    return 1;
    }
    // return the specified type
    public String getString(int columnIndex) throws SQLException {
     	getColumn(columnIndex);
	     return lastColumn;
    }
    public boolean getBoolean(int columnIndex) throws SQLException {
	     getColumn(columnIndex);
	     return Boolean.valueOf(lastColumn).booleanValue();
    }
    public byte getByte(int columnIndex) throws SQLException {
      	byte b = 0;
	     getColumn(columnIndex);
	     try {b = Byte.parseByte(lastColumn);}
	     catch(NumberFormatException e) {}
      	return b;
    }
    public short getShort(int columnIndex) throws SQLException {
     	short s = 0;
     	getColumn(columnIndex);
	     try {s = Short.parseShort(lastColumn);}
	     catch(NumberFormatException e) {}
	     return s;
    }
    public int getInt(int columnIndex) throws SQLException {
	     int i = 0;
	     getColumn(columnIndex);
       try {i = Integer.parseInt(lastColumn);}
	     catch(NumberFormatException e) {}
	     return i;
    }
    public long getLong(int columnIndex) throws SQLException {
	     long l = 0;
	     getColumn(columnIndex);
	     try {l = Long.parseLong(lastColumn);}
       catch(NumberFormatException e) {}
	     return l;
    }
    public float getFloat(int columnIndex) throws SQLException {
	     float f = 0;
	     getColumn(columnIndex);
	     try {f = Float.parseFloat(lastColumn);}
	     catch(NumberFormatException e) {}
	     return f;
    }
    public double getDouble(int columnIndex) throws SQLException {
	     double d = 0;
	     getColumn(columnIndex);
	     try {d = Double.parseDouble(lastColumn);}
	     catch(NumberFormatException e) {}
	     return d;
    }
    // return true, if the last column was null
    public boolean wasNull() throws SQLException { return lastColumn == null; }
    // not supported
    public byte[] getBytes(int columnIndex) throws SQLException { return null; }
    public Date getDate(int columnIndex) throws SQLException { return null; }
    public Time getTime(int columnIndex) throws SQLException { return null; }
    public Timestamp getTimestamp(int columnIndex) throws SQLException { return null; }
    public InputStream getAsciiStream(int columnIndex) throws SQLException { return null; }
    public InputStream getBinaryStream(int columnIndex) throws SQLException { return null; }

    // return the specified type using the name of column
    public String getString(String columnName) throws SQLException {
      	return getString(findColumn(columnName));
    }
    public boolean getBoolean(String columnName) throws SQLException {
	     return getBoolean(findColumn(columnName));
    }
    public byte getByte(String columnName) throws SQLException {
	     return getByte(findColumn(columnName));
    }
    public short getShort(String columnName) throws SQLException {
	     return getShort(findColumn(columnName));
    }
    public int getInt(String columnName) throws SQLException {
	     return getInt(findColumn(columnName));
    }
    public long getLong(String columnName) throws SQLException {
	     return getLong(findColumn(columnName));
    }
    public float getFloat(String columnName) throws SQLException {
	     return getFloat(findColumn(columnName));
    }
    public double getDouble(String columnName) throws SQLException {
	     return getDouble(findColumn(columnName));
    }
    // not supported
    public BigDecimal getBigDecimal(String columnName, int scale) throws SQLException { return null; }
    public byte[] getBytes(String columnName) throws SQLException { return null; }
    public Date getDate(String columnName) throws SQLException { return null; }
    public Time getTime(String columnName) throws SQLException { return null; }
    public Timestamp getTimestamp(String columnName) throws SQLException { return null; }
    public InputStream getAsciiStream(String columnName) throws SQLException { return null; }
    public InputStream getBinaryStream(String columnName) throws SQLException { return null; }

    // warnings are not supported
    public SQLWarning getWarnings() throws SQLException { return null; }
    public void clearWarnings() throws SQLException {}
    // not supported
    public String getCursorName() throws SQLException { return null; }
    public ResultSetMetaData getMetaData() throws SQLException { return null; }

    // not yet implemented
    public Object getObject(int columnIndex) throws SQLException { return null; }
    public Object getObject(String columnName) throws SQLException { return null; }

    public Reader getCharacterStream(int columnIndex) throws SQLException { return null; }
    public Reader getCharacterStream(String columnName) throws SQLException { return null; }
    public BigDecimal getBigDecimal(int columnIndex) throws SQLException { return null; }
    public BigDecimal getBigDecimal(String columnName) throws SQLException { return null; }

    // fetch size and direction
    public void setFetchDirection(int direction) throws SQLException {
      fetchDirection = direction;
    }
    public int getFetchDirection() throws SQLException { return fetchDirection; }
    public void setFetchSize(int size) throws SQLException { fetchSize = size; }
    public int getFetchSize() throws SQLException { return fetchSize; }

    // return type
    public int getType() throws SQLException { return type; }
    // return concurrency
    public int getConcurrency() throws SQLException { return concurrency; }

    // always return false
    public boolean rowUpdated() throws SQLException { return false; }
    public boolean rowInserted() throws SQLException { return false; }
    public boolean rowDeleted() throws SQLException { return false; }

    // update is not supported
    public void updateNull(int columnIndex) throws SQLException {}
    public void updateBoolean(int columnIndex, boolean x) throws SQLException {}
    public void updateByte(int columnIndex, byte x) throws SQLException {}
    public void updateShort(int columnIndex, short x) throws SQLException {}
    public void updateInt(int columnIndex, int x) throws SQLException {}
    public void updateLong(int columnIndex, long x) throws SQLException {}
    public void updateFloat(int columnIndex, float x) throws SQLException {}
    public void updateDouble(int columnIndex, double x) throws SQLException {}
    public void updateBigDecimal(int columnIndex, BigDecimal x) throws SQLException {}
    public void updateString(int columnIndex, String x) throws SQLException {}
    public void updateBytes(int columnIndex, byte[] x) throws SQLException {}
    public void updateDate(int columnIndex, Date x) throws SQLException {}
    public void updateTime(int columnIndex, Time x) throws SQLException {}
    public void updateTimestamp(int columnIndex, Timestamp x) throws SQLException {}
    public void updateAsciiStream(int columnIndex, InputStream x, int length) throws SQLException {}
    public void updateBinaryStream(int columnIndex, InputStream x, int length) throws SQLException {}
    public void updateCharacterStream(int columnIndex, Reader x, int length) throws SQLException {}
    public void updateObject(int columnIndex, Object x, int scale) throws SQLException {}
    public void updateObject(int columnIndex, Object x) throws SQLException {}
    public void updateNull(String columnName) throws SQLException {}
    public void updateBoolean(String columnName, boolean x) throws SQLException {}
    public void updateByte(String columnName, byte x) throws SQLException {}
    public void updateShort(String columnName, short x) throws SQLException {}
    public void updateInt(String columnName, int x) throws SQLException {}
    public void updateLong(String columnName, long x) throws SQLException {}
    public void updateFloat(String columnName, float x) throws SQLException {}
    public void updateDouble(String columnName, double x) throws SQLException {}
    public void updateBigDecimal(String columnName, BigDecimal x) throws SQLException {}
    public void updateString(String columnName, String x) throws SQLException {}
    public void updateBytes(String columnName, byte[] x) throws SQLException {}
    public void updateDate(String columnName, Date x) throws SQLException {}
    public void updateTime(String columnName, Time x) throws SQLException {}
    public void updateTimestamp(String columnName, Timestamp x) throws SQLException {}
    public void updateAsciiStream(String columnName, InputStream x, int length) throws SQLException {}
    public void updateBinaryStream(String columnName, InputStream x, int length) throws SQLException {}
    public void updateCharacterStream(String columnName, Reader reader, int length) throws SQLException {}
    public void updateObject(String columnName, Object x, int scale) throws SQLException {}
    public void updateObject(String columnName, Object x) throws SQLException {}

    public void insertRow() throws SQLException {}
    public void updateRow() throws SQLException {}
    public void deleteRow() throws SQLException {}
    public void refreshRow() throws SQLException {}
    public void cancelRowUpdates() throws SQLException {}
    public void moveToInsertRow() throws SQLException {}
    public void moveToCurrentRow() throws SQLException {}

    // return statement
    public Statement getStatement() throws SQLException { return st; }

    // return null
    public Object getObject(int i, Map map) throws SQLException { return null; }
    public Ref getRef(int i) throws SQLException { return null; }
    public Blob getBlob(int i) throws SQLException { return null; }
    public Clob getClob(int i) throws SQLException { return null; }
    public Array getArray(int i) throws SQLException { return null; }
    public Object getObject(String colName, Map map) throws SQLException { return null; }
    public Ref getRef(String colName) throws SQLException { return null; }
    public Blob getBlob(String colName) throws SQLException { return null; }
    public Clob getClob(String colName) throws SQLException { return null; }
    public Array getArray(String colName) throws SQLException { return null; }
    public Date getDate(int columnIndex, Calendar cal) throws SQLException { return null; }
    public Date getDate(String columnName, Calendar cal) throws SQLException { return null; }
    public Time getTime(int columnIndex, Calendar cal) throws SQLException { return null; }
    public Time getTime(String columnName, Calendar cal) throws SQLException { return null; }
    public Timestamp getTimestamp(int columnIndex, Calendar cal) throws SQLException { return null; }
    public Timestamp getTimestamp(String columnName, Calendar cal) throws SQLException { return null; }
    // deprecated
    public BigDecimal getBigDecimal(int columnIndex, int scale) throws SQLException { return null; }
    public InputStream getUnicodeStream(int columnIndex) throws SQLException { return null; }
    public InputStream getUnicodeStream(String columnName) throws SQLException { return null; }

    public int getSize() {return size;}

    /* formal definitions */
    public void updateArray(String colName, Array x) {}
    public void updateArray(int colName, Array x) {}
    public void updateClob(String x, Clob cb) {}
    public void updateClob(int x, Clob cb) {}
    public void updateBlob(String x, Blob bb) {}
    public void updateBlob(int x, Blob bb) {}
    public void updateRef(String x, Ref bb) {}
    public void updateRef(int x, Ref bb) {}

    public URL getURL(String colName) {return null;}
    public URL getURL(int colName) {return null;}
    public String toString() {
      return getClass()+"[SQL="+sql+",st="+st+",size="+size+",lastCol="+lastColumn+"]";
    }
}